require 'SecureRandom'
require 'fiddle'
require 'fiddle/import'

module D5GA
  # "v":        1,                          # Protocal Version
  # "tid":      GA_TRACK_ID,                # Tracking ID
  # "cid":      str(gaGetCid()),            # Client ID
  # "t":        "event",                    # Hit Type
  # "ec":       eventCategory,              # Event Category
  # "ea":       eventAction                 # Event Action
  # "el":       eventLabel                  # File Size Scale
  # "ev":       eventValue                  # File Size Value
  # "z":        random.randint(0, 10000),   # Random Number

  # category: Normal, Light, DCC_Version, Converter_Version
  # Normal: logo, start, stop, sync, viewOn, viewOff, sendScenes, addLight, setting
  # Light: point, spot, strip, rect, hideLight, showLight
  # DCC_Version: su版本号
  # Converter_Version: 插件版本号

  def D5GA.initial(skpfile,cidfile)
    if D5Converter::ENVIRONMENT<=2 # dev test pre
      @tid="UA-188861875-4"
    elsif D5Converter::ENVIRONMENT==3
      @tid="UA-188861875-5"
    elsif D5Converter::ENVIRONMENT==4
      @tid="UA-188861875-6"
    end

    if File.exist? cidfile
      f = File.new(cidfile,"r")
      @cid = f.sysread(36)
    else
      @cid = SecureRandom.uuid()
      f = File.new(cidfile,"w")
      f.syswrite(@cid)
    end
    if File.exist? skpfile
      fib_seq=[ 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233,
                377, 610, 987, 1597, 2584, 4181, 6765, 10946,
                17711, 28657, 46368, 75025, 121393]
      @ev = File.size(skpfile)/1024/1024
      @el = fib_seq.bsearch { |x|x>=@ev }
      @el= @el==nil ? "inf" : @el
    else
      @ev = 0
      @el = 1
    end
  end

  def D5GA.send_req(eventCategory,eventAction)
    url="https://www.google-analytics.com/collect?v=1"
    url+="&tid=#{@tid}"
    url+="&cid=#{@cid}"
    url+="&t=event"
    url+="&ec=#{eventCategory}"
    url+="&ea=#{eventAction}"
    if @el!=nil and @ev!=nil
      url+="&el=#{@el}"
      url+="&ev=#{@ev}"
      @el = nil
      @ev = nil
    end
    url+="&z=1"
    request = Sketchup::Http::Request.new(url)
    request.start do |a,b|
      if request.status == Sketchup::Http::STATUS_SUCCESS
        #puts "succeed to send request:#{request.url}"
      else
        #puts "#{request.status} can't send request"
      end
    end
  end
end

# 新GA4
module D5GA4
  extend Fiddle::Importer
  dlload(File.join(D5Strs::D5RESOURCE_PATH, "common_utils.dll").encode(Encoding.locale_charmap))
  GAParamsC = struct ['const char* parameters', 'const char* value']
  extern("int InitGAWithoutClientID(int, const wchar_t*)")
  extern("int ReportGA(const char*, const GAParamsC*, size_t, int)")

  def self.InitGA
    InitGAWithoutClientID(D5Converter::ENVIRONMENT,"SketchUp".encode("utf-16le"))
  end

  def self.Report(event,params_array = [],defer = false)
    count = params_array.count
    ga_params_ptr = Fiddle.malloc(count*sizeof(GAParamsC))
    for i in 0..(count-1)
      param = GAParamsC.new(ga_params_ptr + i * sizeof(GAParamsC))
      param.parameters = params_array[i][0].encode(Encoding.locale_charmap)
      param.value = params_array[i][1].encode(Encoding.locale_charmap)
    end
    ReportGA(event.encode(Encoding.locale_charmap),ga_params_ptr,count,defer ? 1 : 0)
  end

  def self.test
    count = 2
    ga_params_ptr = Fiddle.malloc(count*sizeof(GAParamsC))
    #ga_params = []
    param1 = GAParamsC.new(ga_params_ptr)
    param1.parameters = "test_param1".encode(Encoding.locale_charmap)
    param1.value = "".encode(Encoding.locale_charmap)
    #ga_params.push(param1)
    param2 = GAParamsC.new(ga_params_ptr+sizeof(GAParamsC))
    param2.parameters = "test_param2".encode(Encoding.locale_charmap)
    param2.value = "test_value2".encode(Encoding.locale_charmap)
    #ga_params.push(param2)
    ReportGA("test_event".encode(Encoding.locale_charmap),ga_params_ptr,count,0)
  end
end

# Init GA service
D5GA.initial(Sketchup.active_model.path,File.join(D5Strs::D5RESOURCE_PATH,"cid.key"))
D5GA4.InitGA

# Report to GA on loading
D5GA4.Report("P_SketchUp_launch",[["dcc_version",Sketchup.version],["dcc_language",Sketchup.os_language],["p_version",D5Converter::EXTENSION.version]])

# category: Normal, Light
# Normal: logo, start, stop, sync, viewOn, viewOff, sendScenes, addLight, setting
# Light: point, spot, strip, rect, hideLight, showLight
#show the UI of D5Plugin
module D5PluginUI
  def self.toolbar_init
    @main_bar = UI::Toolbar.new('D5 Converter')
    @light_bar = UI::Toolbar.new('D5 Light')
  end

  def self.cmds_init
    @cmdConnect = UI::Command.new('start d5'){
      if $d5Converter_connectionStatus
        D5GA4.Report("P_SketchUp_stop")
        D5GA.send_req("Normal","stop")
        @cmd_imp.stop
      else
        D5GA4.Report("P_SketchUp_start")
        D5GA.send_req("Normal","start")
        D5GA.send_req("DCC_Version",Sketchup.version)
        D5GA.send_req("Converter_Version",D5Converter::EXTENSION.version)
        @cmd_imp.start
      end
    }.set_validation_proc {
      @cmdConnect.status_bar_text = $d5Converter_connectionStatus ? D5Localize.info("TOOLTIP_CONNECT_STOP") : D5Localize.info("TOOLTIP_CONNECT_START")
      $d5Converter_connectionStatus == true ? MF_CHECKED : MF_UNCHECKED
    }

    @cmdSend = UI::Command.new('sync'){
      D5GA4.Report("P_SketchUp_sync")
      D5GA.send_req("Normal","sync")
      @cmd_imp.model_sync
    }.set_validation_proc {
      $d5Converter_connectionStatus == true ? MF_ENABLED : MF_DISABLED | MF_GRAYED
    }

    @cmdSendScenes = UI::Command.new('send scene'){
      D5GA4.Report("P_SketchUp_send_scene")
      D5GA.send_req("Normal","sendScenes")
      @cmd_imp.send_scenes
    }.set_validation_proc {
      $d5Converter_connectionStatus == true ? MF_ENABLED : MF_DISABLED | MF_GRAYED
    }

    @cmdViewSwitch = UI::Command.new('live view switch'){
      if D5ViewAndScene.view_sync_status == true
        D5GA4.Report("P_SketchUp_view-off")
        D5GA.send_req("Normal","viewOff")
        @cmd_imp.view_switch(false)
      else
        D5GA4.Report("P_SketchUp_view-on")
        D5GA.send_req("Normal","viewOn")
        @cmd_imp.view_switch(true)
      end
    }.set_validation_proc {
      @cmdViewSwitch.status_bar_text = D5ViewAndScene.view_sync_status ? D5Localize.info("TOOLTIP_VIEWSWITCH_OFF") : D5Localize.info("TOOLTIP_VIEWSWITCH_ON")

      enable_state = $d5Converter_connectionStatus == true ? MF_ENABLED : MF_DISABLED | MF_GRAYED
      checked_state = D5ViewAndScene.view_sync_status == true ? MF_CHECKED : MF_UNCHECKED
      enable_state | checked_state
    }

    @cmdSetting = UI::Command.new('setting'){
      D5GA4.Report("P_SketchUp_setting")
      D5GA.send_req("Normal","setting")
      @cmd_imp.setting
    }

    @cmdPoint = UI::Command.new('point'){
      D5GA4.Report("P_SketchUp_LightTool_point")
      D5GA.send_req("Light","point")
      Sketchup.active_model.select_tool($d5pointTool)
    }
    @cmdSpot = UI::Command.new('spot'){
      D5GA4.Report("P_SketchUp_LightTool_spot")
      D5GA.send_req("Light","spot")
      Sketchup.active_model.select_tool($d5spotTool)
    }
    @cmdStrip = UI::Command.new('strip'){
      D5GA4.Report("P_SketchUp_LightTool_strip")
      D5GA.send_req("Light","strip")
      Sketchup.active_model.select_tool($d5stripTool)
    }
    @cmdRect = UI::Command.new('rect'){
      D5GA4.Report("P_SketchUp_LightTool_rect")
      D5GA.send_req("Light","rect")
      Sketchup.active_model.select_tool($d5rectTool)
    }

    @is_hidden = false
    @cmdHide = UI::Command.new('hide'){
      if @is_hidden
        D5GA4.Report("P_SketchUp_LightTool_show")
        D5GA.send_req("Light","showLight")
        @is_hidden = false
        @cmd_imp.light_hide(false)
        @cmdHide.status_bar_text=D5Localize.info("LIGHTTIP_HIDE")
      else
        D5GA4.Report("P_SketchUp_LightTool_hide")
        D5GA.send_req("Light","hideLight")
        @is_hidden = true
        @cmd_imp.light_hide(true)
        @cmdHide.status_bar_text=D5Localize.info("LIGHTTIP_SHOW")
      end
    }.set_validation_proc{
      (@is_hidden) ? MF_CHECKED : MF_UNCHECKED
    }

    @cmdLight = UI::Command.new('light'){
      if @light_bar.visible?
        D5GA4.Report("P_SketchUp_LightTool_toolbar_hide")
        D5GA.send_req("Normal","Hide light tools")
        @light_bar.hide
      else
        D5GA4.Report("P_SketchUp_LightTool_toolbar_show")
        D5GA.send_req("Normal","Show light tools")
        @light_bar.show
      end
    }.set_validation_proc{
      @light_bar.visible? ? MF_CHECKED : MF_UNCHECKED
    }
  end

  #complete these information of cmd.
  def self.completeCmdInfo(item,icon,menu_text,status_bar_text)
    # 下面这4个设置之后就不可改变
    item.small_icon = icon
    item.large_icon = icon
    item.menu_text = menu_text
    item.tooltip = menu_text

    # 这个状态栏提示语修改后可以实时生效
    item.status_bar_text = status_bar_text
  end

  def self.cmds_ui
    # example:
    # completeCmdInfo(@cmdViewSwitch, "pictures/Viewport_Connect.png", D5Localize.info("TOOLBAR_VIEWON"), D5Localize.info("TOOLTIP_VIEWON"))
    completeCmdInfo(@cmdConnect,"pictures/Connect.png",D5Localize.info("TOOLBAR_CONNECT"),D5Localize.info("TOOLTIP_CONNECT_START"))
    completeCmdInfo(@cmdSend,"pictures/Sync.png",D5Localize.info("TOOLBAR_SYNC"),D5Localize.info("TOOLTIP_SYNC"))
    completeCmdInfo(@cmdViewSwitch,"pictures/Viewport_Connect.png",D5Localize.info("TOOLBAR_VIEWSWITCH"),D5Localize.info("TOOLTIP_VIEWSWITCH_ON"))
    completeCmdInfo(@cmdSendScenes,"pictures/Scenes_Sync.png",D5Localize.info("TOOLBAR_SENDSCENES"),D5Localize.info("TOOLTIP_SENDSCENES"))
    completeCmdInfo(@cmdSetting,"pictures/Setting.png",D5Localize.info("TOOLBAR_SETTING"),D5Localize.info("TOOLTIP_SETTING"))
    completeCmdInfo(@cmdLight,"pictures/Light_Tools.png",D5Localize.info("TOOLBAR_LIGHT"),D5Localize.info("TOOLTIP_LIGHT"))

    completeCmdInfo(@cmdPoint,"pictures/Point_Light.png",D5Localize.info("LIGHTBAR_POINT"),D5Localize.info("LIGHTTIP_POINT"))
    completeCmdInfo(@cmdSpot,"pictures/Spot_Light.png",D5Localize.info("LIGHTBAR_SPOT"),D5Localize.info("LIGHTTIP_SPOT"))
    completeCmdInfo(@cmdStrip,"pictures/Strip_Light.png",D5Localize.info("LIGHTBAR_STRIP"),D5Localize.info("LIGHTTIP_STRIP"))
    completeCmdInfo(@cmdRect,"pictures/Rect_Light.png",D5Localize.info("LIGHTBAR_RECT"),D5Localize.info("LIGHTTIP_RECT"))
    completeCmdInfo(@cmdHide,"pictures/Hide.png",D5Localize.info("LIGHTBAR_HIDE"),D5Localize.info("LIGHTTIP_HIDE"))
  end

  def self.add_cmds_to_toolbar
    @main_bar.add_item(@cmdConnect)
    @main_bar.add_separator
    @main_bar.add_item(@cmdSend)
    @main_bar.add_item(@cmdViewSwitch)
    @main_bar.add_item(@cmdSendScenes)
    @main_bar.add_separator
    @main_bar.add_item(@cmdLight)
    @main_bar.add_separator
    @main_bar.add_item(@cmdSetting)

    # https://forums.sketchup.com/t/how-to-get-a-toolbar-to-appear-on-first-load-of-a-plugin/33830
    if @main_bar.get_last_state == TB_NEVER_SHOWN
      @main_bar.show
      UI.start_timer(0.1, false) {
        @main_bar.show
      }
    else
      @main_bar.restore
      UI.start_timer(0.1, false) {
        @main_bar.restore
      }
    end

    @light_bar.add_item(@cmdPoint)
    @light_bar.add_item(@cmdSpot)
    @light_bar.add_item(@cmdStrip)
    @light_bar.add_item(@cmdRect)
    @light_bar.add_separator
    @light_bar.add_item(@cmdHide)

    # https://forums.sketchup.com/t/how-to-get-a-toolbar-to-appear-on-first-load-of-a-plugin/33830
    if @light_bar.get_last_state == TB_NEVER_SHOWN
      @light_bar.show
      UI.start_timer(0.1, false) {
        @light_bar.show
      }
    else
      @light_bar.restore
      UI.start_timer(0.1, false) {
        @light_bar.restore
      }
    end
  end

  # once
  def self.add_to_menu
    d5Converter_menu = UI.menu('Extensions')
    d5Converter_submenu = d5Converter_menu.add_submenu('D5 Converter')
    d5Converter_submenu.add_item(@cmdConnect)
    d5Converter_submenu.add_item(@cmdSend)
    d5Converter_submenu.add_item(@cmdViewSwitch)
    d5Converter_submenu.add_item(@cmdSendScenes)
    d5Converter_submenu.add_item(@cmdSetting)
    d5Converter_submenu.add_item(@cmdLight)
  end


  def self.bind_implement cmd_imp
    @cmd_imp = cmd_imp
  end
  
  def self.show
    self.toolbar_init
    self.cmds_init
    self.cmds_ui
    self.add_cmds_to_toolbar
    self.add_to_menu
  end
end

